@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2006 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2025 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This library is free software; you can redistribute it and/or modify  *
@@ -28,7 +28,6 @@
 
 #if WORDS_BIGENDIAN
 # define RIFF_TYPE_DLS	0x444C5320
-# define LIST_TYPE_INFO	0x494E464F
 # define LIST_TYPE_WVPL	0x7776706C
 # define LIST_TYPE_DWPL 0x6477706C  ///< Seen on some files instead of a wvpl list chunk.
 # define LIST_TYPE_WAVE	0x77617665
@@ -42,17 +41,10 @@
 # define CHUNK_ID_IARL	0x4941524C
 # define CHUNK_ID_IART	0x49415254
 # define CHUNK_ID_ICMS	0x49434D53
-# define CHUNK_ID_ICMT	0x49434D54
-# define CHUNK_ID_ICOP	0x49434F50
-# define CHUNK_ID_ICRD	0x49435244
-# define CHUNK_ID_IENG	0x49454E47
 # define CHUNK_ID_IGNR	0x49474E52
 # define CHUNK_ID_IKEY	0x494B4559
 # define CHUNK_ID_IMED	0x494D4544
-# define CHUNK_ID_INAM	0x494E414D
-# define CHUNK_ID_IPRD	0x49505244
 # define CHUNK_ID_ISBJ	0x4953424A
-# define CHUNK_ID_ISFT	0x49534654
 # define CHUNK_ID_ISRC	0x49535243
 # define CHUNK_ID_ISRF	0x49535246
 # define CHUNK_ID_ITCH	0x49544348
@@ -68,9 +60,10 @@
 # define CHUNK_ID_COLH	0x636F6C68
 # define CHUNK_ID_ARTL	0x6172746C
 # define CHUNK_ID_ART2	0x61727432
+# define CHUNK_ID_XFIL	0x7866696C
+# define CHUNK_ID_DOXF	0x646F7866
 #else  // little endian
 # define RIFF_TYPE_DLS	0x20534C44
-# define LIST_TYPE_INFO	0x4F464E49
 # define LIST_TYPE_WVPL	0x6C707677
 # define LIST_TYPE_DWPL 0x6C707764  ///< Seen on some files instead of a wvpl list chunk.
 # define LIST_TYPE_WAVE	0x65766177
@@ -84,17 +77,10 @@
 # define CHUNK_ID_IARL	0x4C524149
 # define CHUNK_ID_IART	0x54524149
 # define CHUNK_ID_ICMS	0x534D4349
-# define CHUNK_ID_ICMT	0x544D4349
-# define CHUNK_ID_ICOP	0x504F4349
-# define CHUNK_ID_ICRD	0x44524349
-# define CHUNK_ID_IENG	0x474E4549
 # define CHUNK_ID_IGNR	0x524E4749
 # define CHUNK_ID_IKEY	0x59454B49
 # define CHUNK_ID_IMED	0x44454D49
-# define CHUNK_ID_INAM	0x4D414E49
-# define CHUNK_ID_IPRD	0x44525049
 # define CHUNK_ID_ISBJ	0x4A425349
-# define CHUNK_ID_ISFT	0x54465349
 # define CHUNK_ID_ISRC	0x43525349
 # define CHUNK_ID_ISRF	0x46525349
 # define CHUNK_ID_ITCH	0x48435449
@@ -110,9 +96,11 @@
 # define CHUNK_ID_COLH	0x686C6F63
 # define CHUNK_ID_ARTL	0x6C747261
 # define CHUNK_ID_ART2	0x32747261
+# define CHUNK_ID_XFIL	0x6C696678
+# define CHUNK_ID_DOXF	0x66786F64
 #endif // WORDS_BIGENDIAN
 
-#define WAVE_FORMAT_PCM			0x0001
+#define DLS_WAVE_FORMAT_PCM			0x0001
 
 //TODO: no support for conditional chunks <cdl> yet
 
@@ -120,6 +108,8 @@
 namespace DLS {
 
     typedef std::string String;
+    typedef RIFF::progress_t progress_t;
+    typedef RIFF::file_offset_t file_offset_t;
 
     /** Quadtuple version number ("major.minor.release.build"). */
     struct version_t {
@@ -138,7 +128,7 @@ namespace DLS {
     };
 
     /** Connection Sources */
-    typedef enum {
+    enum conn_src_t {
         // Modulator Sources
         conn_src_none            = 0x0000,
         conn_src_lfo             = 0x0001,
@@ -161,10 +151,10 @@ namespace DLS {
         conn_src_rpn0            = 0x0100,
         conn_src_rpn1            = 0x0101,
         conn_src_rpn2            = 0x0102
-    } conn_src_t;
+    };
 
     /** Connection Destinations */
-    typedef enum {
+    enum conn_dst_t {
         // Generic Destinations
         conn_dst_none             = 0x0000,
         conn_dst_gain             = 0x0001,
@@ -206,28 +196,47 @@ namespace DLS {
         // Filter Destinations
         conn_dst_filter_cutoff    = 0x0500,
         conn_dst_filter_q         = 0x0501
-    } conn_dst_t;
+    };
 
     /** Connection Transforms */
-    typedef enum {
+    enum conn_trn_t {
         conn_trn_none    = 0x0000,
         conn_trn_concave = 0x0001,
         conn_trn_convex  = 0x0002,
         conn_trn_switch  = 0x0003
-    } conn_trn_t;
+    };
 
     /** Lower and upper limit of a range. */
     struct range_t {
         uint16_t low;  ///< Low value of range.
         uint16_t high; ///< High value of range.
+
+        inline bool operator< (const range_t& other) const {
+            if (low < other.low) return true;
+            if (low > other.low) return false;
+            return high < other.high;
+        }
+
+        inline bool operator== (const range_t& other) const {
+            return low == other.low && high == other.high;
+        }
+
+        inline bool overlaps(uint16_t scalar) const {
+            return low <= scalar && scalar <= high;
+        }
+
+        inline bool overlaps(const range_t& other) const {
+            return overlaps(other.low) || overlaps(other.high) ||
+                   other.overlaps(low) || other.overlaps(high);
+        }
     };
 
     /** Defines Sample Loop Points. */
     struct sample_loop_t {
-        uint32_t Size;
-        uint32_t LoopType;
-        uint32_t LoopStart;
-        uint32_t LoopLength;
+        uint32_t Size;       ///< For internal usage only: usually reflects exactly @c sizeof(sample_loop_t), otherwise if the value is larger then the DLS format was extended!
+        uint32_t LoopType;   ///< Defines how the waveform samples will be looped (appropriate loop types for the gig format are defined by gig::loop_type_t).
+        uint32_t LoopStart;  ///< The start value specifies the offset (in sample points) in the waveform data of the first sample point to be played in the loop.
+        uint32_t LoopLength; ///< Length of the looping area (in sample points).
     };
 
     // just symbol prototyping
@@ -258,36 +267,90 @@ namespace DLS {
                 uint16_t transform;
                 uint32_t scale;
             };
-            Connection() {};
+            Connection() {}
             void Init(conn_block_t* Header);
             conn_block_t ToConnBlock();
-            virtual ~Connection() {};
+            virtual ~Connection() {}
             friend class Articulation;
     };
 
+    /** @brief Abstract base class for all classes using RIFF::Chunks for persistency.
+     *
+     * This abstract base class defines the general interface for all classes
+     * which are using RIFF::Chunks to actually load and store their data
+     * persistently from/to disk.
+     */
+    class Storage {
+    public:
+        /** @brief Apply object's changes to the respective RIF::Chunks.
+         *
+         * This abstract interface method is intended to be implemented by the
+         * deriving classes by updating the respective RIFF chunks associated
+         * with the object such that those RIFF chunks reflect the object's
+         * current data (i.e. object's current member variables). So the purpose
+         * of this method is to prepare for saving the object's current state
+         * persistently to the actual RIFF file.
+         *
+         * After returning from this method the changes are just scheduled to be
+         * saved to the RIFF file, it is required to call File::Save()
+         * subsequently to make the changes actually persistent on file level.
+         *
+         * Usually there is no need to call this method directly from an
+         * application. This method is called automatically by libgig if one of
+         * the respective API methods is called to save the file persistently
+         * to disk (i.e. DLS::File::Save() or gig::File::Save()).
+         *
+         * @param pProgress - callback function for progress notification
+         */
+        virtual void UpdateChunks(progress_t* pProgress) = 0;
+
+        /** @brief Remove all RIFF chunks associated with this object.
+         *
+         * This abstract interface method is intended to be implemented by the
+         * deriving classes by removing every RIFF::Chunk the deriving overall
+         * object is using to store the object in the final RIFF::File. In other
+         * words: the intention is to remove the deriving class(es)'s object
+         * persistently from the currently open file.
+         *
+         * Note that the RIFF::Chunks deletions is just scheduled after
+         * returning from this method. You have to call File::Save() to make
+         * these changes persistent on file level.
+         *
+         * Usually there is no need to call this method directly from an
+         * application. This method is called automatically by libgig if one of
+         * the respective API methods is called to remove the respective object
+         * persistently (i.e. File::DeleteInstrument() or File::DeleteSample()).
+         */
+        virtual void DeleteChunks() = 0;
+    };
+
     /** Provides access to the defined connections used for the synthesis model. */
-    class Articulation {
+    class Articulation : public Storage {
         public:
             Connection*  pConnections; ///< Points to the beginning of a <i>Connection</i> array.
             uint32_t     Connections;  ///< Reflects the number of Connections.
 
             Articulation(RIFF::Chunk* artl);
             virtual ~Articulation();
-            virtual void UpdateChunks();
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void DeleteChunks() OVERRIDE;
         protected:
             RIFF::Chunk* pArticulationCk;
             uint32_t     HeaderSize;
     };
 
     /** Abstract base class for classes that provide articulation information (thus for <i>Instrument</i> and <i>Region</i> class). */
-    class Articulator {
+    class Articulator : public Storage {
         public:
             Articulator(RIFF::List* ParentList);
-            Articulation* GetFirstArticulation();
-            Articulation* GetNextArticulation();
-            virtual void  UpdateChunks();
+            Articulation* GetArticulation(size_t pos);
+            Articulation* GetFirstArticulation() LIBGIG_DEPRECATED_API("Use GetArticulation() instead.");
+            Articulation* GetNextArticulation() LIBGIG_DEPRECATED_API("Use GetArticulation() instead.");
+            virtual void  UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void  DeleteChunks() OVERRIDE;
+            virtual void  CopyAssign(const Articulator* orig);
         protected:
-            typedef std::list<Articulation*> ArticulationList;
+            typedef std::vector<Articulation*> ArticulationList;
             RIFF::List*                 pParentList;
             ArticulationList*           pArticulations;
             ArticulationList::iterator  ArticulationsIterator;
@@ -297,7 +360,7 @@ namespace DLS {
     };
 
     /** Optional information for DLS files, instruments, samples, etc. */
-    class Info {
+    class Info : public Storage {
         public:
             String Name;             ///< <INAM-ck>. Stores the title of the subject of the file, such as, Seattle From Above.
             String ArchivalLocation; ///< <IARL-ck>. Indicates where the subject of the file is stored.
@@ -316,26 +379,40 @@ namespace DLS {
             String SourceForm;       ///< <ISRF-ck>. Identifies the original form of the material that was digitized, such as record, sampling CD, TV sound track. This is not neccessarily the same as <i>Medium</i>.
             String Commissioned;     ///< <ICMS-ck>. Lists the name of the person or organization that commissioned the subject of the file, e.g., Pope Julian II.
             String Subject;          ///< <ISBJ-ck>. Describes the contents of the file.
-            bool UseFixedLengthStrings; ///< Set this to true if the info strings should be stored with a fixed length format. This is used for gig files, not for ordinary DLS files.
+            bool UseFixedLengthStrings LIBGIG_DEPRECATED_API("Not used anymore, use SetFixedStringLengths() instead."); ///< @deprecated Not used anymore, use SetFixedStringLengths() instead.
+
+            struct string_length_t {
+                uint32_t chunkId;
+                int      length;
+            };
 
             Info(RIFF::List* list);
+            void SetFixedStringLengths(const string_length_t* lengths);
             virtual ~Info();
-            virtual void UpdateChunks();
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void DeleteChunks() OVERRIDE;
+            virtual void CopyAssign(const Info* orig);
         private:
-            RIFF::List* pResourceListChunk;
+            RIFF::List*            pResourceListChunk;
+            const string_length_t* pFixedStringLengths; ///< List of IDs and string lengths for strings that should be stored in a fixed length format. This is used for gig files, not for ordinary DLS files.
 
             static void LoadString(uint32_t ChunkID, RIFF::List* lstINFO, String& s);
-            static void SaveString(uint32_t ChunkID, RIFF::List* lstINFO, const String& s, const String& sDefault, bool bUseFixedLengthStrings, int size);
+            void SaveString(uint32_t ChunkID, RIFF::List* lstINFO, const String& s, const String& sDefault);
     };
 
     /** Abstract base class which encapsulates data structures which all DLS resources are able to provide. */
-    class Resource {
+    class Resource : public Storage {
         public:
             Info*    pInfo;  ///< Points (in any case) to an <i>Info</i> object, providing additional, optional infos and comments.
             dlsid_t* pDLSID; ///< Points to a <i>dlsid_t</i> structure if the file provided a DLS ID else is <i>NULL</i>.
 
-            Resource* GetParent() { return pParent; };
-            virtual void UpdateChunks();
+            Resource* GetParent() { return pParent; }
+            const Resource* GetParent() const { return pParent; }
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void DeleteChunks() OVERRIDE;
+            void GenerateDLSID();
+            static void GenerateDLSID(dlsid_t* pDLSID);
+            virtual void CopyAssign(const Resource* orig);
         protected:
             Resource* pParent;
             RIFF::List* pResourceList;
@@ -345,17 +422,22 @@ namespace DLS {
     };
 
     /** Abstract base class which provides mandatory informations about sample players in general. */
-    class Sampler {
+    class Sampler : public Storage {
         public:
             uint8_t        UnityNote;
             int16_t        FineTune;
-            int32_t        Gain;
+            int32_t        Gain; ///< @deprecated Don't alter directly, use SetGain() instead!
             bool           NoSampleDepthTruncation;
             bool           NoSampleCompression;
             uint32_t       SampleLoops;  ///< Reflects the number of sample loops.
             sample_loop_t* pSampleLoops; ///< Points to the beginning of a sample loop array, or is NULL if there are no loops defined.
 
-            virtual void UpdateChunks();
+            void AddSampleLoop(sample_loop_t* pLoopDef);
+            void DeleteSampleLoop(sample_loop_t* pLoopDef);
+            virtual void SetGain(int32_t gain);
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void DeleteChunks() OVERRIDE;
+            virtual void CopyAssign(const Sampler* orig);
         protected:
             RIFF::List*    pParentList;
             uint32_t       uiHeaderSize;
@@ -374,31 +456,35 @@ namespace DLS {
      */
     class Sample : public Resource {
         public:
-            uint16_t      FormatTag;             ///< Format ID of the waveform data (should be WAVE_FORMAT_PCM for DLS1 compliant files, this is also the default value if Sample was created with Instrument::AddSample()).
+            uint16_t      FormatTag;             ///< Format ID of the waveform data (should be DLS_WAVE_FORMAT_PCM for DLS1 compliant files, this is also the default value if Sample was created with Instrument::AddSample()).
             uint16_t      Channels;              ///< Number of channels represented in the waveform data, e.g. 1 for mono, 2 for stereo (defaults to 1=mono if Sample was created with Instrument::AddSample() previously).
             uint32_t      SamplesPerSecond;      ///< Sampling rate at which each channel should be played (defaults to 44100 if Sample was created with Instrument::AddSample() previously).
             uint32_t      AverageBytesPerSecond; ///< The average number of bytes per second at which the waveform data should be transferred (Playback software can estimate the buffer size using this value).
             uint16_t      BlockAlign;            ///< The block alignment (in bytes) of the waveform data. Playback software needs to process a multiple of <i>BlockAlign</i> bytes of data at a time, so the value of <i>BlockAlign</i> can be used for buffer alignment.
             uint16_t      BitDepth;              ///< Size of each sample per channel (only if known sample data format is used, 0 otherwise).
-            unsigned long SamplesTotal;          ///< Reflects total number of sample points (only if known sample data format is used, 0 otherwise), do not bother to change this value, it will not be saved.
+            file_offset_t SamplesTotal;          ///< Reflects total number of sample points (only if known sample data format is used, 0 otherwise), do not bother to change this value, it will not be saved.
             uint          FrameSize;             ///< Reflects the size (in bytes) of one single sample point (only if known sample data format is used, 0 otherwise). <b>Caution:</b> with the current version of libgig you have to upate this field by yourself whenever you change one of the following fields: Channels, BitDepth ! Ignoring this might lead to undesired behavior when i.e. calling Resize(), SetPos(), Write() or Read().
 
             void*         LoadSampleData();
             void          ReleaseSampleData();
-            unsigned long GetSize();
-            void          Resize(int iNewSize);
-            unsigned long SetPos(unsigned long SampleCount, RIFF::stream_whence_t Whence = RIFF::stream_start);
-            unsigned long Read(void* pBuffer, unsigned long SampleCount);
-            unsigned long Write(void* pBuffer, unsigned long SampleCount);
-            virtual void  UpdateChunks();
+            file_offset_t GetSize() const;
+            void          Resize(file_offset_t NewSize);
+            file_offset_t SetPos(file_offset_t SampleCount, RIFF::stream_whence_t Whence = RIFF::stream_start);
+            file_offset_t Read(void* pBuffer, file_offset_t SampleCount);
+            file_offset_t Write(void* pBuffer, file_offset_t SampleCount);
+            virtual void  UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void  DeleteChunks() OVERRIDE;
+            virtual void  CopyAssign(const Sample* orig);
+
         protected:
             RIFF::List*   pWaveList;
             RIFF::Chunk*  pCkData;
             RIFF::Chunk*  pCkFormat;
-            unsigned long ulWavePoolOffset;  // needed for comparison with the wave pool link table, thus the link to instruments
+            file_offset_t ullWavePoolOffset;  // needed for comparison with the wave pool link table, thus the link to instruments
 
-            Sample(File* pFile, RIFF::List* waveList, unsigned long WavePoolOffset);
+            Sample(File* pFile, RIFF::List* waveList, file_offset_t WavePoolOffset);
             virtual ~Sample();
+            void CopyAssignCore(const Sample* orig);
             friend class File;
             friend class Region; // Region has to compare the wave pool offset to get its sample
     };
@@ -406,7 +492,7 @@ namespace DLS {
     /** Defines <i>Region</i> information of an <i>Instrument</i>. */
     class Region : public Resource, public Articulator, public Sampler {
         public:
-            range_t     KeyRange;
+            range_t     KeyRange; ///< @deprecated Only read, don't write! Use SetKeyRange() instead.
             range_t     VelocityRange;
             uint16_t    KeyGroup;
             uint16_t    Layer;
@@ -418,7 +504,10 @@ namespace DLS {
 
             Sample*     GetSample();
             void        SetSample(Sample* pSample);
-            virtual void UpdateChunks();
+            virtual void SetKeyRange(uint16_t Low, uint16_t High);
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void DeleteChunks() OVERRIDE;
+            virtual void CopyAssign(const Region* orig);
         protected:
             RIFF::List* pCkRegion;
             uint32_t    WavePoolTableIndex; // index in the wave pool table to the sample wave this region is linked to
@@ -441,13 +530,17 @@ namespace DLS {
             uint32_t MIDIProgram;    ///< Specifies the MIDI Program Change Number this Instrument should be assigned to.
             uint32_t Regions;        ///< Reflects the number of <i>Region</i> defintions this Instrument has.
 
-            Region*  GetFirstRegion();
-            Region*  GetNextRegion();
+            size_t   CountRegions();
+            Region*  GetRegionAt(size_t pos);
+            Region*  GetFirstRegion() LIBGIG_DEPRECATED_API("Use GetRegionAt() instead.");
+            Region*  GetNextRegion() LIBGIG_DEPRECATED_API("Use GetRegionAt() instead.");
             Region*  AddRegion();
             void     DeleteRegion(Region* pRegion);
-            virtual void UpdateChunks();
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void DeleteChunks() OVERRIDE;
+            virtual void CopyAssign(const Instrument* orig);
         protected:
-            typedef std::list<Region*> RegionList;
+            typedef std::vector<Region*> RegionList;
             struct midi_locale_t {
                 uint32_t bank;
                 uint32_t instrument;
@@ -458,9 +551,13 @@ namespace DLS {
             RegionList::iterator RegionsIterator;
 
             Instrument(File* pFile, RIFF::List* insList);
+            void CopyAssignCore(const Instrument* orig);
             virtual void LoadRegions();
             virtual ~Instrument();
             friend class File;
+            friend class Region;
+        private:
+            void MoveRegion(Region* pSrc, Region* pDst);
     };
 
     /** Parses DLS Level 1 and 2 compliant files and provides abstract access to the data. */
@@ -471,21 +568,28 @@ namespace DLS {
 
             File();
             File(RIFF::File* pRIFF);
-            Sample*     GetFirstSample();     ///< Returns a pointer to the first <i>Sample</i> object of the file, <i>NULL</i> otherwise.
-            Sample*     GetNextSample();      ///< Returns a pointer to the next <i>Sample</i> object of the file, <i>NULL</i> otherwise.
+            String      GetFileName();
+            void        SetFileName(const String& name);
+            Sample*     GetSample(size_t index);
+            Sample*     GetFirstSample() LIBGIG_DEPRECATED_API("Use GetSample() instead.");
+            Sample*     GetNextSample() LIBGIG_DEPRECATED_API("Use GetSample() instead.");
             Sample*     AddSample();
+            size_t      CountSamples();
             void        DeleteSample(Sample* pSample);
-            Instrument* GetFirstInstrument(); ///< Returns a pointer to the first <i>Instrument</i> object of the file, <i>NULL</i> otherwise.
-            Instrument* GetNextInstrument();  ///< Returns a pointer to the next <i>Instrument</i> object of the file, <i>NULL</i> otherwise.
+            Instrument* GetInstrument(size_t index);
+            Instrument* GetFirstInstrument() LIBGIG_DEPRECATED_API("Use GetInstrument() instead.");
+            Instrument* GetNextInstrument() LIBGIG_DEPRECATED_API("Use GetInstrument() instead.");
             Instrument* AddInstrument();
             void        DeleteInstrument(Instrument* pInstrument);
-            virtual void UpdateChunks();
-            virtual void Save(const String& Path);
-            virtual void Save();
+            RIFF::File* GetRiffFile();
+            RIFF::File* GetExtensionFile(int index);
+            virtual void UpdateChunks(progress_t* pProgress) OVERRIDE;
+            virtual void Save(const String& Path, progress_t* pProgress = NULL);
+            virtual void Save(progress_t* pProgress = NULL);
             virtual ~File();
         protected:
-            typedef std::list<Sample*>     SampleList;
-            typedef std::list<Instrument*> InstrumentList;
+            typedef std::vector<Sample*>     SampleList;
+            typedef std::vector<Instrument*> InstrumentList;
 
             RIFF::File*              pRIFF;
             std::list<RIFF::File*>   ExtensionFiles;
@@ -498,9 +602,11 @@ namespace DLS {
             uint32_t*                pWavePoolTable;
             uint32_t*                pWavePoolTableHi;
             bool                     b64BitWavePoolOffsets;
+            bool                     bOwningRiff; ///< If @c true then @c pRIFF was implicitly allocated by this class and hence pRIFF will automatically be freed by the @c DLS::File destructor in that case.
 
             virtual void LoadSamples();
             virtual void LoadInstruments();
+            virtual void UpdateFileOffsets();
             void __ensureMandatoryChunksExist();
             friend class Region; // Region has to look in the wave pool table to get its sample
         private:
@@ -508,11 +614,20 @@ namespace DLS {
             void __UpdateWavePoolTable();
     };
 
-    /** Will be thrown whenever a DLS specific error occurs while trying to access a DLS File. */
+    /**
+     * Will be thrown whenever a DLS specific error occurs while trying to
+     * access a DLS File. Note: In your application you should better catch
+     * for RIFF::Exception rather than this one, except you explicitly want
+     * to catch and handle DLS::Exception and RIFF::Exception independently,
+     * which usually shouldn't be necessary though.
+     */
     class Exception : public RIFF::Exception {
         public:
-            Exception(String Message);
-            void PrintMessage();
+            Exception(String format, ...);
+            Exception(String format, va_list arg);
+            void PrintMessage() const;
+        protected:
+            Exception();
     };
 
     String libraryName();
