@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
- *   Copyright (C) 2014 Christian Schoenebeck                              *
- *                      <cuse@users.sourceforge.net>                       *
+ *   Copyright (C) 2014 - 2025 Christian Schoenebeck                       *
+ *                             <cuse@users.sourceforge.net>                *
  *                                                                         *
  *   This library is part of libgig.                                       *
  *                                                                         *
@@ -55,7 +55,7 @@ namespace Korg {
     template<unsigned int SZ>
     inline String readText(RIFF::Chunk* ck) {
         char buf[SZ+1] = {};
-        int n = ck->Read(buf, SZ, 1);
+        int n = (int) ck->Read(buf, SZ, 1);
         if (n != SZ)
             throw Exception("Premature end while reading text field");
         String s = buf;
@@ -412,7 +412,7 @@ namespace Korg {
     }
 
     int KMPInstrument::GetRegionCount() const {
-        return regions.size();
+        return (int) regions.size();
     }
 
     bool KMPInstrument::Use2ndStart() const {
@@ -437,7 +437,7 @@ namespace Korg {
     Exception::Exception(String Message) : RIFF::Exception(Message) {
     }
 
-    void Exception::PrintMessage() {
+    void Exception::PrintMessage() const {
         std::cout << "Korg::Exception: " << Message << std::endl;
     }
     
