@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2015 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2025 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This program is part of libgig.                                       *
@@ -67,14 +67,14 @@ static void printUsage() {
     cout << endl;
     cout << "   -v               Print version and exit." << endl;
     cout << endl;
-    cout << "   --verbose        Print additional informations while converting." << endl;
+    cout << "   --verbose        Print additional information while converting." << endl;
     cout << endl;
     cout << "Read `man gig2stereo' for details." << endl;
     cout << endl;
 }
 
 static string programRevision() {
-    string s = "$Revision: 2706 $";
+    string s = "$Revision: 4245 $";
     return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
 }
 
@@ -89,6 +89,7 @@ static bool isGigFileName(const string path) {
 }
 
 static bool endsWith(const string& haystack, const string& needle, bool caseSensitive) {
+    if (haystack.size() < needle.size()) return false;
     const string sub = haystack.substr(haystack.size() - needle.size(), needle.size());
     return (caseSensitive) ? (sub == needle) : (!strcasecmp(sub.c_str(), needle.c_str()));
 }
@@ -203,7 +204,7 @@ static string stripAudioChannelFromName(const string& s) {
  *                            their main sample characteristics match, if set
  *                            to true this sanity check will be skipped
  * @param verbosity - verbosity level, defines whether and how much additional
- *                    informations to be printed to the console while doing the
+ *                    information to be printed to the console while doing the
  *                    conversion (0 .. 2)
  */
 static bool convertFileToStereo(const string path, bool keep, bool forceReplace, bool skipIncompatible, int verbose) {
@@ -221,8 +222,8 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
         map<SamplePair, vector<DimRgnPair> > samplePairs;
         map<gig::Sample*, SamplePair> samplePairsByL;
         map<gig::Sample*, SamplePair> samplePairsByR;
-        for (gig::Instrument* instr = gig.GetFirstInstrument(); instr; instr = gig.GetNextInstrument()) {
-            for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+        for (size_t i = 0; gig::Instrument* instr = gig.GetInstrument(i); ++i) {
+            for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
                 int bits = 0;
                 for (int d = 0; d < 8; ++d) {
                     if (rgn->pDimensionDefinitions[d].dimension == gig::dimension_samplechannel) {
@@ -321,16 +322,16 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
                     OPTIONAL_SKIP_CHECK();
                 }
                 if (pSampleL->LoopStart != pSampleR->LoopStart) {
-                    cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop start! Skipping!\n";
-                    continue; // skip to convert this sample pair
+                    cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop start!";
+                    OPTIONAL_SKIP_CHECK();
                 }
                 if (pSampleL->LoopEnd != pSampleR->LoopEnd) {
-                    cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop end! Skipping!\n";
-                    continue; // skip to convert this sample pair
+                    cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop end!";
+                    OPTIONAL_SKIP_CHECK();
                 }
                 if (pSampleL->LoopSize != pSampleR->LoopSize) {
-                    cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop size! Skipping!\n";
-                    continue; // skip to convert this sample pair
+                    cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop size!";
+                    OPTIONAL_SKIP_CHECK();
                 }
                 if (pSampleL->LoopPlayCount != pSampleR->LoopPlayCount) {
                     cerr << "WARNING: Sample pair ['" << pSampleL->pInfo->Name << "', '" << pSampleR->pInfo->Name << "'] with different loop play count!";
@@ -344,7 +345,7 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
             // add the new stereo sample to an appropriate sample group
             const string groupName = stripWhiteSpace(stripAudioChannelFromName(pSampleL->GetGroup()->Name)) + " STEREO";
             gig::Group* pGroup = NULL;
-            for (gig::Group* gr = gig.GetFirstGroup(); gr; gr = gig.GetNextGroup()) {
+            for (size_t g = 0; gig::Group* gr = gig.GetGroup(g); ++g) {
                 if (gr->Name == groupName) {
                     pGroup = gr;
                     break;
@@ -485,8 +486,8 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
                 for (int iChan = 0; iChan < 2; ++iChan) {
                     gig::Sample* pMonoSample = pMonoSamples[iChan];
                     // iterate through all instruments and replace all references of that sample
-                    for (gig::Instrument* instr = gig.GetFirstInstrument(); instr; instr = gig.GetNextInstrument()) {
-                        for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+                    for (size_t i = 0; gig::Instrument* instr = gig.GetInstrument(i); ++i) {
+                        for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
                             for (int dr = 0; dr < rgn->DimensionRegions; ++dr) {
                                 if (!rgn->pDimensionRegions[dr])
                                     continue;
@@ -523,8 +524,8 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
 
                     // iterate through all instruments and check if that
                     // old mono sample is still referenced somewhere
-                    for (gig::Instrument* instr = gig.GetFirstInstrument(); instr; instr = gig.GetNextInstrument()) {
-                        for (gig::Region* rgn = instr->GetFirstRegion(); rgn; rgn = instr->GetNextRegion()) {
+                    for (size_t i = 0; gig::Instrument* instr = gig.GetInstrument(i); ++i) {
+                        for (size_t r = 0; gig::Region* rgn = instr->GetRegionAt(r); ++r) {
                             for (int dr = 0; dr < rgn->DimensionRegions; ++dr) {
                                 if (!rgn->pDimensionRegions[dr])
                                     continue;
@@ -544,7 +545,8 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
                         iDeletedSamples++;
 
                         // if deleted sample's group is now empty, delete the group
-                        if (!pGroup->GetFirstSample()) gig.DeleteGroup(pGroup);
+                        if (!pGroup->GetSample(0))
+                            gig.DeleteGroup(pGroup);
                     }
 
                     nextSample:;
@@ -567,7 +569,7 @@ static bool convertFileToStereo(const string path, bool keep, bool forceReplace,
         e.PrintMessage();
         return false; // error
     } catch (...) {
-        cerr << "Unknown exception occured while trying to convert file." << endl;
+        cerr << "Unknown exception occurred while trying to convert file." << endl;
         return false; // error
     }
     return true; // success
