@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2019 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2025 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This program is part of libgig.                                       *
@@ -195,36 +195,31 @@ void PrintFileInformation(gig::File* gig) {
 }
 
 void PrintGroups(gig::File* gig) {
-    int groups = 0;
     cout << "ALL defined Groups:" << endl;
-    for (gig::Group* pGroup = gig->GetFirstGroup(); pGroup; pGroup = gig->GetNextGroup()) {
-        groups++;
+    for (size_t g = 0; gig::Group* pGroup = gig->GetGroup(g); ++g) {
         string name = pGroup->Name;
         if (name == "") name = "<NO NAME>";
         else            name = '\"' + name + '\"';
-        cout << "    Group " << groups << ")" << endl;
+        cout << "    Group " << (g+1) << ")" << endl;
         cout << "        Name: " << name << endl;
     }
 }
 
 void PrintSamples(gig::File* gig) {
-    int samples = 0;
     cout << "ALL Available Samples (as there might be more than referenced by Instruments):" << endl;
-    PubSample* pSample = (PubSample*) gig->GetFirstSample();
-    while (pSample) {
-        samples++;
+    for (size_t s = 0; PubSample* pSample = (PubSample*) gig->GetSample(s); ++s) {
         // determine sample's name
         string name = pSample->pInfo->Name;
         if (name == "") name = "<NO NAME>";
         else            name = '\"' + name + '\"';
         // determine group this sample belongs to
-        int iGroup = 1;
-        for (gig::Group* pGroup = gig->GetFirstGroup(); pGroup; pGroup = gig->GetNextGroup(), iGroup++) {
+        size_t g;
+        for (g = 0; gig::Group* pGroup = gig->GetGroup(g); ++g) {
             if (pGroup == pSample->GetGroup()) break;
         }
         // print sample info
-        cout << "    Sample " << samples << ") " << name << ", ";
-        cout << "Group " << iGroup << ", ";
+        cout << "    Sample " << (s+1) << ") " << name << ", ";
+        cout << "Group " << (g+1) << ", ";
         cout << pSample->SamplesPerSecond << "Hz, " << pSample->Channels << " Channels, " << pSample->Loops << " Loops";
         if (pSample->Loops) {
             cout << " (Type: ";
@@ -257,7 +252,6 @@ void PrintSamples(gig::File* gig) {
             fflush(stdout);
         }
 #endif
-        pSample = (PubSample*) gig->GetNextSample();
     }
 }
 
@@ -277,15 +271,12 @@ void PrintScripts(gig::File* gig) {
 }
 
 void PrintInstruments(gig::File* gig) {
-    int instruments = 0;
     cout << "Available Instruments:" << endl;
-    gig::Instrument* pInstrument = gig->GetFirstInstrument();
-    while (pInstrument) {
-        instruments++;
+    for (size_t i = 0; gig::Instrument* pInstrument = gig->GetInstrument(i); ++i) {
         string name = pInstrument->pInfo->Name;
         if (name == "") name = "<NO NAME>";
         else            name = '\"' + name + '\"';
-        cout << "    Instrument " << instruments << ") " << name << ", ";
+        cout << "    Instrument " << (i+1) << ") " << name << ", ";
 
         cout << " MIDIBank=" << pInstrument->MIDIBank << ", MIDIProgram=" << pInstrument->MIDIProgram << endl;
 
@@ -297,28 +288,21 @@ void PrintInstruments(gig::File* gig) {
         }
 
         PrintRegions(pInstrument);
-
-        pInstrument = gig->GetNextInstrument();
     }
 }
 
 void PrintInstrumentNamesOnly(gig::File* gig) {
-    int instruments = 0;
-    gig::Instrument* pInstrument = gig->GetFirstInstrument();
-    for (; pInstrument; pInstrument = gig->GetNextInstrument()) {
-        instruments++;
+    for (size_t i = 0; gig::Instrument* pInstrument = gig->GetInstrument(i); ++i) {
         string name = pInstrument->pInfo->Name;
         if (name == "") name = "<NO NAME>";
         else            name = '\"' + name + '\"';
-        cout << "Instrument " << instruments << ") " << name << endl;
+        cout << "Instrument " << (i+1) << ") " << name << endl;
     }
 }
 
 void PrintRegions(gig::Instrument* instr) {
-    int iRegion = 1;
-    gig::Region* pRegion = instr->GetFirstRegion();
-    while (pRegion) {
-        cout << "        Region " << iRegion++ << ") ";
+    for (size_t r = 0; gig::Region* pRegion = instr->GetRegionAt(r); ++r) {
+        cout << "        Region " << (r+1) << ") ";
         gig::Sample* pSample = pRegion->GetSample();
         if (pSample) {
             cout << "Sample: ";
@@ -461,8 +445,6 @@ void PrintRegions(gig::Instrument* instr) {
         }
 
         PrintDimensionRegions(pRegion);
-
-        pRegion = instr->GetNextRegion();
     }
 }
 
@@ -587,14 +569,14 @@ bool VerifyFile(gig::File* _gig) {
 
     cout << "Verifying samples ... " << flush;
     std::map<int,_FailedSample> failedSamples;
-    int iTotal = 0;
-    for (gig::Sample* pSample = gig->GetFirstSample(); pSample; pSample = gig->GetNextSample(), ++iTotal) {
+    const size_t iTotal = gig->CountSamples();
+    for (size_t s = 0; gig::Sample* pSample = gig->GetSample(s); ++s) {
         uint32_t crc; // will be set to the actually now calculated checksum
         if (!pSample->VerifyWaveData(&crc)) {
             _FailedSample failed;
             failed.sample = pSample;
             failed.calculatedCRC = crc;
-            failedSamples[iTotal] = failed;
+            failedSamples[s] = failed;
         }
     }
     if (failedSamples.empty()) {
@@ -632,7 +614,7 @@ void RebuildChecksumTable(gig::File* _gig) {
 }
 
 string Revision() {
-    string s = "$Revision: 3623 $";
+    string s = "$Revision: 4245 $";
     return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
 }
 
