@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2013 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2025 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This program is part of libgig.                                       *
@@ -190,6 +190,13 @@ static std::string getLoopTypeText(gig::loop_type_t type) {
     }
 }
 
+static string replacePathSeparators(string s) {
+    for (int i = 0; i < s.size(); ++i)
+        if (s[i] == '/' || s[i] == '\\')
+            s[i] = '-';
+    return s;
+}
+
 void ExtractSamples(gig::File* gig, char* destdir, OrderMap* ordered) {
 #if !HAVE_SNDFILE // use libaudiofile
     hAFlib = NULL;
@@ -198,24 +205,19 @@ void ExtractSamples(gig::File* gig, char* destdir, OrderMap* ordered) {
     uint8_t* pWave  = NULL;
     int* pIntWave = NULL;
     long BufferSize = 0;
-    int samples     = 0;
     gig::buffer_t decompressionBuffer;
     decompressionBuffer.Size = 0;
     cout << "Seeking for available samples..." << flush;
-    gig::Sample* pSample = gig->GetFirstSample();
+    gig->GetSample(0);
     cout << "OK" << endl << flush;
-    while (pSample) {
-        samples++;
-        if (ordered) {
-            if ((*ordered)[samples] == false) {
-                pSample = gig->GetNextSample();
-                continue;
-            }
-        }
-        string name = pSample->pInfo->Name;
+    for (size_t s = 0; gig::Sample* pSample = gig->GetSample(s); ++s) {
+        if (ordered && (*ordered)[s+1] == false)
+            continue;
+
+        string name = replacePathSeparators(pSample->pInfo->Name);
         string filename = destdir;
         if (filename[filename.size() - 1] != '/') filename += "/";
-        filename += ToString(samples);
+        filename += ToString(s+1);
         filename += "_";
         if (name == "") {
             name = "(NO NAME)";
@@ -229,7 +231,7 @@ void ExtractSamples(gig::File* gig, char* destdir, OrderMap* ordered) {
         filename += ".wav";
         if (pSample->Compressed) cout << "Decompressing ";
         else                     cout << "Extracting ";
-        cout << "Sample " << samples << ") " << name << " (" << pSample->BitDepth <<"Bits, " << pSample->SamplesPerSecond << "Hz, " << pSample->Channels << " Channels, " << pSample->SamplesTotal << " Samples";
+        cout << "Sample " << (s+1) << ") " << name << " (" << pSample->BitDepth <<"Bits, " << pSample->SamplesPerSecond << "Hz, " << pSample->Channels << " Channels, " << pSample->SamplesTotal << " Samples";
         if (pSample->Loops > 0) {
             cout << ", LoopType "  << getLoopTypeText(pSample->LoopType)
                  << ", LoopStart " << pSample->LoopStart
@@ -313,8 +315,6 @@ void ExtractSamples(gig::File* gig, char* destdir, OrderMap* ordered) {
             pSample->ReleaseSampleData(); // free wave from RAM
         }
         else cout << "Failed to load sample data." << endl;
-
-        pSample = gig->GetNextSample();
     }
     gig::Sample::DestroyDecompressionBuffer(decompressionBuffer);
 #if USE_DISK_STREAMING
@@ -434,7 +434,7 @@ void closeAFlib() {
 #endif // !HAVE_SNDFILE
 
 string Revision() {
-    string s = "$Revision: 2577 $";
+    string s = "$Revision: 4244 $";
     return s.substr(11, s.size() - 13); // cut dollar signs, spaces and CVS macro keyword
 }
 
